## $Id: browser.py 14254 2016-11-03 07:04:45Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import hashlib
import grok
from zope.interface import Interface
from zope.component import queryAdapter
from kofacustom.nigeria.interswitch.browser import (
    InterswitchPaymentRequestWebservicePageStudent,
    InterswitchPaymentRequestWebservicePageApplicant,
    InterswitchPaymentVerifyWebservicePageApplicant,
    InterswitchPaymentVerifyWebservicePageStudent,
    InterswitchPageStudent, InterswitchPageApplicant,
    )
from waeup.kwarapoly.students.interfaces import ICustomStudentOnlinePayment
from waeup.kwarapoly.applicants.interfaces import ICustomApplicantOnlinePayment
from waeup.kwarapoly.interfaces import MessageFactory as _

PRODUCT_ID = '3930'
SITE_NAME = 'kwarapoly.waeup.org'
PROVIDER_ACCT = '1010764827'
PROVIDER_BANK_ID = '117'
PROVIDER_ITEM_NAME = 'BT Education'
INSTITUTION_NAME = 'KwaraPoly'
CURRENCY = '566'
GATEWAY_AMT = 300.0
#QUERY_URL = 'https://webpay.interswitchng.com/paydirect/services/TransactionQueryURL.aspx'
#QUERY_URL = 'https://testwebpay.interswitchng.com/test_paydirect/services/TransactionQueryURL.aspx'

POST_ACTION = 'https://webpay.interswitchng.com/paydirect/webpay/pay.aspx'
#POST_ACTION = 'https://testwebpay.interswitchng.com/test_paydirect/webpay/pay.aspx'

HOST = 'webpay.interswitchng.com'
#HOST = 'testwebpay.interswitchng.com'

URL = '/paydirect/services/TransactionQueryWs.asmx'
#URL = '/test_paydirect/services/TransactionQueryWs.asmx'
httplib.HTTPConnection.debuglevel = 0

#All special payments moved to GTBank on 15-10-2015
SPECIAL_PAYMENT_PARAMS = {
    'certificate': ('111', 200.0, 300.0, '0106259811', '10'),
    'state_result': ('112', 200.0, 300.0, '0106259811', '10'),
    'transcript_local': ('113', 200.0, 300.0, '0106259811', '10'),
    'transcript_foreign': ('114', 200.0, 300.0, '0106259811', '10'),
    'ver_result': ('115', 200.0, 300.0, '0106259811', '10'),
    'change_course': ('116', 200.0, 300.0, '0106259811', '10'),
    'change_inst': ('117', 200.0, 300.0, '0106259811', '10'),
    'jamb_reject': ('118', 200.0, 300.0, '0106259811', '10'),
    'cert_of_cert': ('119', 200.0, 300.0, '0106259811', '10'),
    'ref_let': ('120', 200.0, 300.0, '0106259811', '10'),
    'proc_cert': ('121', 200.0, 300.0, '0106259811', '10'),
    'loss_idcard': ('122', 6.0, 9.0, '0106259811', '10'),
    'loss_examcard': ('123', 3.0, 4.5, '0106259811', '10'),
    'loss_result': ('124', 200.0, 300.0, '0106259811', '10'),
    'loss_receipt': ('125', 200.0, 300.0, '0106259811', '10'),
    'loss_clearance': ('126', 200.0, 300.0, '0106259811', '10'),
    'conv_brochure': ('127', 200.0, 300.0, '0106259811', '10'),
    'hnd_certificate': ('128', 200.0, 300.0, '0106259811', '10'),
    'hnd_state_result': ('129', 200.0, 300.0, '0106259811', '10'),
    'hnd_transcript_local': ('130', 200.0, 300.0, '0106259811', '10'),
    'hnd_transcript_foreign': ('131', 200.0, 300.0, '0106259811', '10'),
    'staff_loss_idcard': ('132', 6.0, 9.0, '0106259811', '10'),
    'hnd_conv_brochure': ('133', 200.0, 300.0, '0106259811', '10'),
    'pgd_state_result': ('134', 200.0, 300.0, '0106259811', '10'),
    'pgd_conv_brochure': ('135', 200.0, 300.0, '0106259811', '10'),
    'pgd_certificate': ('136', 200.0, 300.0, '0106259811', '10'),
    'log_book': ('137', 3.0, 4.5, '0106259811', '10'),
    'pgd_transcript_foreign': ('138', 200.0, 300.0, '0106259811', '10'),
    'pgd_transcript_local': ('139', 200.0, 300.0, '0106259811', '10'),
    'jamb_regularization': ('140', 200.0, 300.0, '0106259811', '10'),
    'utme_registration': ('142', 6.0, 9.0, '0106259811', '10'),
    'utme_cbt': ('143', 6.0, 9.0, '0106259811', '10'),
    'nysc_id_card': ('144', 6.0, 9.0, '0106259811', '10'),
    'ijmb_result': ('147', 200.0, 300.0, '0106259811', '10'),
    }

class CustomInterswitchPageStudent(InterswitchPageStudent):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    grok.template('student_goto_interswitch')
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    product_id = PRODUCT_ID
    mac = 'E6BA6CBBA9AF2871EE25C32C8D57C98895B9B001DC5B9CB2C463E2A9BDA44A3F1260C8A364F33789CDF74CB3EE7E6EF5D94F48D3AF7B727E75D97F07618DFA6D'

    def update(self):
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        student = self.student
        xmldict = self.xmldict
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        # Institution data
        xmldict['institution_item_name'] = self.category
        xmldict['institution_name'] = INSTITUTION_NAME
        xmldict['institution_amt'] = 0.0
        dalash_amt = 200.0
        provider_amt = 300.0
        gateway_amt = GATEWAY_AMT
        self.pay_item_id = '000'
        xmldict['institution_acct'] = "0106259811"
        xmldict['institution_bank_id'] = "10"
        if self.context.p_category == 'schoolfee' \
            or self.context.p_category.startswith('carryover'):
            self.pay_item_id = '101'
            dalash_amt = 1800.0
            provider_amt = 1200.0
            if not student.current_mode.endswith('_we'):
                if xmldict['faculty'] in ('CPGS',):
                    xmldict['institution_acct'] = "0106259811"   
                    xmldict['institution_bank_id'] = '10'
                elif xmldict['faculty'] in ('IAS',):
                    xmldict['institution_acct'] = "0106259811"  
                    xmldict['institution_bank_id'] = '10'
                    self.pay_item_id = '107'
                elif xmldict['faculty'] in ('IES',):
                    xmldict['institution_acct'] = "0106259811" 
                    xmldict['institution_bank_id'] = '10'
                    self.pay_item_id = '109'
                elif xmldict['faculty'] in ('IFMS',):
                    xmldict['institution_acct'] = "0106259811" 
                    xmldict['institution_bank_id'] = '10'
                    self.pay_item_id = '106'
                elif xmldict['faculty'] in ('IOT',):
                    xmldict['institution_acct'] = "0106259811"    
                    xmldict['institution_bank_id'] = '10'
                    self.pay_item_id = '110'
                elif xmldict['faculty'] in ('IICT',):
                    xmldict['institution_acct'] = "0106259811" 
                    xmldict['institution_bank_id'] = '10'
                    self.pay_item_id = '108'
            else:
                xmldict['institution_acct'] = "0106259811"  
                xmldict['institution_bank_id'] = '10'
                #Introduce a pay_item_id for part-time for the first time on this portal
                self.pay_item_id = '145'
        elif self.context.p_category in SPECIAL_PAYMENT_PARAMS.keys():
            self.pay_item_id = SPECIAL_PAYMENT_PARAMS[self.context.p_category][0]
            dalash_amt = SPECIAL_PAYMENT_PARAMS[self.context.p_category][1]
            provider_amt = SPECIAL_PAYMENT_PARAMS[self.context.p_category][2]
            xmldict['institution_acct'] = SPECIAL_PAYMENT_PARAMS[self.context.p_category][3]
            xmldict['institution_bank_id'] = SPECIAL_PAYMENT_PARAMS[self.context.p_category][4]
            if self.context.amount_auth <= 2030.0:
                gateway_amt = round(0.015 * self.context.amount_auth, 2)
        elif 'maintenance' in self.context.p_category:
            self.pay_item_id = '102'
            xmldict['institution_acct'] = "0106259811"
            xmldict['institution_bank_id'] = '10'
            dalash_amt = 0.0
            provider_amt = 0.0
        elif 'clearance' in self.context.p_category:
            self.pay_item_id = '146'
            xmldict['institution_acct'] = "0106259811"
            xmldict['institution_bank_id'] = '10'
            if self.context.amount_auth <= 5077.0:
                gateway_amt = round(0.015 * self.context.amount_auth, 2)
        else:
            xmldict['institution_acct'] = "0000000000000"
            xmldict['institution_bank_id'] = '0'

        xmldict['dalash_amt'] = 100 * dalash_amt
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - provider_amt -
                gateway_amt - dalash_amt)

        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()

        # Interswitch amount is not part of the xml data

        if 'maintenance' in self.context.p_category:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        else:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="Dalash" item_amt="%(dalash_amt)d" bank_id="117" acct_num="1013196791" />
<item_detail item_id="3" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = gateway_amt
        self.context.thirdparty_amt = dalash_amt
        return

class CustomInterswitchPageApplicant(InterswitchPageApplicant):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomApplicantOnlinePayment)
    grok.template('applicant_goto_interswitch')
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = '103'
    product_id = PRODUCT_ID
    mac = 'E6BA6CBBA9AF2871EE25C32C8D57C98895B9B001DC5B9CB2C463E2A9BDA44A3F1260C8A364F33789CDF74CB3EE7E6EF5D94F48D3AF7B727E75D97F07618DFA6D'

    def update(self):
        error = self.init_update()
        if error:
            self.flash(error, type='danger')
            self.redirect(self.url(self.context, '@@index'))
            return
        xmldict = {}
        provider_amt = 300.0
        dalash_amt = 200.0
        gateway_amt = GATEWAY_AMT
        xmldict['institution_acct'] = '00000000000'
        xmldict['institution_bank_id'] = '00'
        if self.applicant.applicant_id.startswith('hnd'):
            xmldict['institution_acct'] = '0106259811'
            xmldict['institution_bank_id'] = '10'
        elif self.applicant.applicant_id.startswith('nd'):
            xmldict['institution_acct'] = '0106259811'
            xmldict['institution_bank_id'] = '10'
        elif self.applicant.applicant_id.startswith('rmd'):
            xmldict['institution_acct'] = '0106259811'
            xmldict['institution_bank_id'] = '10'
        elif self.applicant.applicant_id.startswith('pgft'):
            xmldict['institution_acct'] = '0106259811'
            xmldict['institution_bank_id'] = '10'
        elif self.applicant.applicant_id.startswith('putme'):
            xmldict['institution_acct'] = '0106259811'
            xmldict['institution_bank_id'] = '10'
            self.pay_item_id = '104'
        elif self.applicant.applicant_id.startswith('pre'):
            xmldict['institution_acct'] = '0106259811'
            xmldict['institution_bank_id'] = '10'
            provider_amt = 262.5
            dalash_amt = 0.0
            gateway_amt = 37.5
            self.pay_item_id = '104'
        elif self.applicant.applicant_id.startswith('special'):
            if self.context.p_category in SPECIAL_PAYMENT_PARAMS.keys():
                self.pay_item_id = SPECIAL_PAYMENT_PARAMS[self.context.p_category][0]
                dalash_amt = SPECIAL_PAYMENT_PARAMS[self.context.p_category][1]
                provider_amt = SPECIAL_PAYMENT_PARAMS[self.context.p_category][2]
                xmldict['institution_acct'] = SPECIAL_PAYMENT_PARAMS[self.context.p_category][3]
                xmldict['institution_bank_id'] = SPECIAL_PAYMENT_PARAMS[self.context.p_category][4]
                if self.context.amount_auth <= 2030.0:
                    gateway_amt = round(0.015 * self.context.amount_auth, 2)
        xmldict['dalash_amt'] = 100 * dalash_amt
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_amt'] = 100 * provider_amt
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        xmldict['institution_amt'] = 100 * (
            self.context.amount_auth - provider_amt - gateway_amt - dalash_amt)
        xmldict['institution_item_name'] = self.category
        xmldict['institution_name'] = INSTITUTION_NAME

        hashargs = (
            self.context.p_id +
            PRODUCT_ID +
            self.pay_item_id +
            str(int(self.amount_auth)) +
            self.site_redirect_url +
            self.mac)
        self.hashvalue = hashlib.sha512(hashargs).hexdigest()

        # Interswitch amount is not part of the xml data

        xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="Dalash" item_amt="%(dalash_amt)d" bank_id="117" acct_num="1013196791" />
<item_detail item_id="3" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        if self.applicant.applicant_id.startswith('pre'):
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s">
<item_detail item_id="1" item_name="%(institution_item_name)s" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = gateway_amt
        self.context.thirdparty_amt = dalash_amt
        return

class CustomInterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL

class CustomInterswitchPaymentVerifyWebservicePageStudent(
    InterswitchPaymentVerifyWebservicePageStudent):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL

class CustomInterswitchPaymentRequestWebservicePageApplicant(
    InterswitchPaymentRequestWebservicePageApplicant):
    """Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL

class CustomInterswitchPaymentVerifyWebservicePageApplicant(
    InterswitchPaymentVerifyWebservicePageApplicant):
    """Payment verify view for the CollegePAY gateway
    """
    grok.context(ICustomApplicantOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
